#include "resizearray.h"
#include <stdlib.h>

//Skapar en ny struktur som hanterar en förstorbar array,
//dynamiskt allokerad. Parametern size anger startstorleken. 
//Returnerar en pekare till denna struktur
prArray newRArray(int size) {
    //malloc allokerar (reserverar) bara minne, utan att nollställa
    prArray ra = (prArray) malloc(sizeof(rArray));
    //calloc nollställer allt minne som allokeras
    //notera även skillnaden i argumenten till malloc och calloc
    ra->data = (char *) calloc((size_t)size, sizeof(char));
    ra->size = size;
    return ra;
}

void resize(prArray ra, int requestedSize) {
    //Tillåt inte nya storlekar som är mindre än dubbla den gamla
    //för att undvika att kopiera data många gånger
    int newSize = (requestedSize < ra->size * 2) ? ra->size * 2 : requestedSize;
    
    //Obs! kopieringen nedanför utförs enklast med realloc, nedanstående gör 
    //det som realloc hade gjort, fast manuellt.
    
    //Allokera minne för den nya, större arrayen
    char *newData = (char*) calloc((size_t)newSize, sizeof(char));
    int i;
    //Kopiera gamla data till den nya arrayen 
    for (i = 0; i < ra->size; i++) {
        newData[i] = ra->data[i];
    }
    free(ra->data);
    ra->data = newData;
    ra->size = newSize;
}

//Sätter elementet på indexet givet av argumentet index till
//värdet givet av argumentet data. Om index är för stort,
//förstoras arrayen. 
void setelem(prArray ra, int index, char data) {
    if (index > ra->size - 1) {
        resize(ra, index + 1);
    }
    ra->data[index] = data;
}

//Hämtar elementet på indexet givet av argumentet index. 
//Om index är för stort, förstoras arrayen (och en nolla returneras).
char getelem(prArray ra, int index) {
    if (index > ra->size -1) {
        resize(ra, index + 1);
    }
    return ra->data[index];
}

//Lämnar tillbaka allokerat minne
void deleteRArray(prArray ra) {
    //Ordningen här spelar roll. 
    free(ra->data);
    free(ra);
    
}
